@@ -1,4 +1,4 @@
-# Copyright (c) 2014 Pavel 'Blane' Tuchin
+# Copyright (c) 2021 Pavel 'Blane' Tuchin
 # This file is part of pynetdicom2, released under a modified MIT license.
 #    See the file license.txt included with this distribution.
 """
@@ -8,15 +8,26 @@ to and from binary formats as specified in PS 3.7 of DICOM standard.
 """
 
 import struct
-from . import _dicom
+
+from pydicom import uid
 
 
 class MaximumLengthSubItem(object):
+    """Represents sub-item described in PS 3.8 D.1 Maximum Length Negotiation
+
+    Note that item is used in both A-ASSOCIATE-RQ and A-ASSOCIATE-AC PDUs.
+
+    :ivar reserved: this reserved field shall be sent with a value 00H but not
+                    tested to this value when received.
+    :ivar item_length: item length, in the case of this item it's fixed to 0x0004
+    :ivar maximum_length_received: P-DATA-TF PDUs size limit
+    """
     item_type = 0x51
     item_format = struct.Struct('>B B H I')
 
     def __init__(self, maximum_length_received, reserved=0x00,
                  item_length=0x0004):
+        # type: (int,int,int) -> None
         self.reserved = reserved  # unsigned byte
         self.item_length = item_length  # unsigned short
         self.maximum_length_received = maximum_length_received  # unsigned int
@@ -39,8 +50,11 @@ class MaximumLengthSubItem(object):
         return 0x08
 
     def encode(self):
-        return self.item_format.pack(self.item_type, self.reserved,
-                                     self.item_length,
+        """Encodes itself into binary form
+
+        :return: binary representation of an item
+        """
+        return self.item_format.pack(self.item_type, self.reserved, self.item_length,
                                      self.maximum_length_received)
 
     @classmethod
@@ -51,17 +65,25 @@ class MaximumLengthSubItem(object):
         :param stream: raw data stream
         :return decoded maximum length sub-item
         """
-        _, reserved, item_length, \
-            maximum_length_received = cls.item_format.unpack(stream.read(8))
+        _, reserved, item_length, maximum_length_received = cls.item_format.unpack(stream.read(8))
         return cls(reserved=reserved, item_length=item_length,
                    maximum_length_received=maximum_length_received)
 
 
 class ImplementationClassUIDSubItem(object):
+    """Represents sub-item described in PS 3.8 D.3.3.2 Implementation Identification Notification
+
+    Note that item is used in both A-ASSOCIATE-RQ and A-ASSOCIATE-AC PDUs.
+
+    :ivar reserved: this reserved field shall be sent with a value 00H but not
+                    tested to this value when received.
+    :ivar implementation_class_uid: This variable field shall contain the Implementation-class-uid
+    """
     item_type = 0x52
     header = struct.Struct('>B B H')
 
     def __init__(self, implementation_class_uid, reserved=0x00):
+        # type: (uid.UID,int) -> None
         self.reserved = reserved  # unsigned byte
         self.implementation_class_uid = implementation_class_uid  # string
 
@@ -72,30 +94,56 @@ class ImplementationClassUIDSubItem(object):
 
     @property
     def item_length(self):
+        """Calculates item length
+
+        :return: item length
+        """
         return len(self.implementation_class_uid)
 
     @property
     def total_length(self):
+        """Returns total item length, including header.
+
+        :return: total item length
+        """
         return 4 + self.item_length
 
     def encode(self):
-        return b''.join([self.header.pack(self.item_type, self.reserved,
-                                          self.item_length),
+        """Encodes itself into binary form
+
+        :return: binary representation of an item
+        """
+        return b''.join([self.header.pack(self.item_type, self.reserved, self.item_length),
                          self.implementation_class_uid.encode()])
 
     @classmethod
     def decode(cls, stream):
+        """Decodes Implementation Class UID sub-item from data stream
+
+        :rtype ImplementationClassUIDSubItem
+        :param stream: raw data stream
+        :return decoded maximum length sub-item
+        """
         _, reserved, item_length = cls.header.unpack(stream.read(4))
-        implementation_class_uid = _dicom.UID(stream.read(item_length).decode())
-        return cls(reserved=reserved,
-                   implementation_class_uid=implementation_class_uid)
+        implementation_class_uid = uid.UID(stream.read(item_length).decode())
+        return cls(reserved=reserved, implementation_class_uid=implementation_class_uid)
 
 
 class ImplementationVersionNameSubItem(object):
+    """Represents sub-item described in PS 3.8 D.3.3.2 Implementation Identification Notification
+
+    Note that item is used in both A-ASSOCIATE-RQ and A-ASSOCIATE-AC PDUs.
+
+    :ivar reserved: this reserved field shall be sent with a value 00H but not
+                    tested to this value when received.
+    :ivar implementation_version_name: This variable field shall contain the
+                                       Implementation-version-name
+    """
     item_type = 0x55
     header = struct.Struct('> B B H')
 
     def __init__(self, implementation_version_name, reserved=0x00):
+        # type: (str,int) -> None
         self.reserved = reserved  # unsigned byte
         self.implementation_version_name = implementation_version_name  # string
 
@@ -107,19 +155,36 @@ class ImplementationVersionNameSubItem(object):
 
     @property
     def item_length(self):
+        """Calculates item length
+
+        :return: item length
+        """
         return len(self.implementation_version_name)
 
     @property
     def total_length(self):
+        """Returns total item length, including header.
+
+        :return: total item length
+        """
         return 4 + self.item_length
 
     def encode(self):
-        return b''.join([self.header.pack(self.item_type, self.reserved,
-                                          self.item_length),
+        """Encodes itself into binary form
+
+        :return: binary representation of an item
+        """
+        return b''.join([self.header.pack(self.item_type, self.reserved, self.item_length),
                          self.implementation_version_name.encode()])
 
     @classmethod
     def decode(cls, stream):
+        """Decodes Implementation Version Name sub-item from data stream
+
+        :rtype ImplementationVersionNameSubItem
+        :param stream: raw data stream
+        :return decoded Implementation Version Name sub-item
+        """
         _, reserved, item_length = cls.header.unpack(stream.read(4))
         implementation_version_name = stream.read(item_length).decode()
         return cls(implementation_version_name=implementation_version_name,
@@ -127,11 +192,23 @@ class ImplementationVersionNameSubItem(object):
 
 
 class AsynchronousOperationsWindowSubItem(object):
+    """Represents sub-item described in PS 3.8 D.3.3.3 Asynchronous Operations (And Sub-Operations)
+    Window Negotiation
+
+    Note that item is used in both A-ASSOCIATE-RQ and A-ASSOCIATE-AC PDUs.
+
+    :ivar reserved: this reserved field shall be sent with a value 00H but not
+                    tested to this value when received.
+    :ivar item_length: item length, in the case of this item it's fixed to 0x0004
+    :ivar max_num_ops_invoked: This field shall contain the Maximum-number-operations-invoked
+    :ivar max_num_ops_performed: This field shall contain the Maximum-number-operations-performed
+    """
     item_type = 0x53
     item_format = struct.Struct('>B B H H H')
 
     def __init__(self, max_num_ops_invoked, max_num_ops_performed,
                  reserved=0x00, item_length=0x0004):
+        # type: (int,int,int,int) -> None
         self.reserved = reserved  # unsigned byte
         self.item_length = item_length  # unsigned short
         self.max_num_ops_invoked = max_num_ops_invoked  # unsigned short
@@ -146,15 +223,29 @@ class AsynchronousOperationsWindowSubItem(object):
 
     @property
     def total_length(self):
+        """Returns total item length, including header.
+
+        :return: total item length
+        """
         return 4 + self.item_length
 
     def encode(self):
+        """Encodes itself into binary form
+
+        :return: binary representation of an item
+        """
         return self.item_format.pack(self.item_type, self.reserved,
                                      self.item_length, self.max_num_ops_invoked,
                                      self.max_num_ops_performed)
 
     @classmethod
     def decode(cls, stream):
+        """Decodes Asynchronous Operations Window sub-item from data stream
+
+        :rtype AsynchronousOperationsWindowSubItem
+        :param stream: raw data stream
+        :return decoded Asynchronous Operations Window sub-item
+        """
         _, reserved, item_length, max_num_ops_invoked, \
             max_num_ops_performed = cls.item_format.unpack(stream.read(8))
         return cls(reserved=reserved, item_length=item_length,
@@ -163,11 +254,21 @@ class AsynchronousOperationsWindowSubItem(object):
 
 
 class ScpScuRoleSelectionSubItem(object):
+    """Represents sub-item described in PS 3.8 D.3.3.4 SCP/SCU Role Selection Negotiation
+
+    Note that item is used in both A-ASSOCIATE-RQ and A-ASSOCIATE-AC PDUs.
+
+    :ivar reserved: this reserved field shall be sent with a value 00H but not
+                    tested to this value when received.
+    :ivar sop_class_uid: SOP Class UID or Meta SOP Class UID
+    :ivar scu_role: 0 - non support of the SCU role, 1 - support of the SCU role
+    :ivar scp_role: 0 - non support of the SCP role, 1 - support of the SCP role.
+    """
     item_type = 0x54
     header = struct.Struct('>B B H H')
 
-    def __init__(self, sop_class_uid, scu_role,
-                 scp_role, reserved=0x00):
+    def __init__(self, sop_class_uid, scu_role, scp_role, reserved=0x00):
+        # type: (uid.UID,int,int,int) -> None
         self.reserved = reserved  # unsigned byte 0x00
         self.sop_class_uid = sop_class_uid  # string
         self.scu_role = scu_role  # unsigned byte
@@ -181,13 +282,25 @@ class ScpScuRoleSelectionSubItem(object):
 
     @property
     def item_length(self):
+        """Calculates item length
+
+        :return: item length
+        """
         return 4 + len(self.sop_class_uid)
 
     @property
     def total_length(self):
+        """Returns total item length, including header.
+
+        :return: total item length
+        """
         return 4 + self.item_length
 
     def encode(self):
+        """Encodes itself into binary form
+
+        :return: binary representation of an item
+        """
         return b''.join(
             [self.header.pack(self.item_type, self.reserved, self.item_length,
                               len(self.sop_class_uid)),
@@ -196,8 +309,14 @@ class ScpScuRoleSelectionSubItem(object):
 
     @classmethod
     def decode(cls, stream):
-        _, reserved, item_length, uid_length = cls.header.unpack(stream.read(6))
-        sop_class_uid = _dicom.UID(stream.read(uid_length).decode())
+        """Decodes SCP/SCU Role Selection sub-item from data stream
+
+        :rtype ScpScuRoleSelectionSubItem
+        :param stream: raw data stream
+        :return decoded SCP/SCU Role Selection sub-item
+        """
+        _, reserved, _, uid_length = cls.header.unpack(stream.read(6))
+        sop_class_uid = uid.UID(stream.read(uid_length).decode())
         scu_role, scp_role = struct.unpack('B B', stream.read(2))
         return cls(reserved=reserved, sop_class_uid=sop_class_uid,
                    scu_role=scu_role, scp_role=scp_role)
@@ -209,15 +328,16 @@ class SOPClassExtendedNegotiationSubItem(object):
 
     Note that item is used in both A-ASSOCIATE-RQ and A-ASSOCIATE-AC PDUs.
 
-    :param sop_class_uid: service's SOP Class UID
-    :param app_info: application information specific to Service Class
-    :param reserved: reserved field, defaults to 0x00. In most cases you
-                     should not change it's value or check it
+    :ivar sop_class_uid: service's SOP Class UID
+    :ivar app_info: application information specific to Service Class
+    :ivar reserved: reserved field, defaults to 0x00. In most cases you
+                    should not change it's value or check it
     """
     item_type = 0x56
     header = struct.Struct('>B B H H')
 
     def __init__(self, sop_class_uid, app_info, reserved=0x00):
+        # type: (uid.UID,bytes,int) -> None
         """Initializes new sub item instance"""
         self.reserved = reserved
         self.sop_class_uid = sop_class_uid
@@ -233,7 +353,7 @@ class SOPClassExtendedNegotiationSubItem(object):
 
     @property
     def total_length(self):
-        """Returns total item length, including header.
+        """Returns total item length, including the header.
 
         :return: total item length
         """
@@ -258,11 +378,10 @@ class SOPClassExtendedNegotiationSubItem(object):
         :return: new sub-item
         """
         _, reserved, item_length, uid_length = cls.header.unpack(stream.read(6))
-        sop_class_uid = _dicom.UID(stream.read(uid_length).decode())
+        sop_class_uid = uid.UID(stream.read(uid_length).decode())
         app_info_length = item_length - uid_length
         app_info = stream.read(app_info_length)
-        return cls(reserved=reserved, sop_class_uid=sop_class_uid,
-                   app_info=app_info)
+        return cls(reserved=reserved, sop_class_uid=sop_class_uid, app_info=app_info)
 
 
 class UserIdentityNegotiationSubItem(object):
@@ -272,28 +391,51 @@ class UserIdentityNegotiationSubItem(object):
     Passes user identification information based on login (and password) or
     kerberos service ticket
 
-    :param primary_field: user name or kerberos ticket depending on
-                          value of `user_identity_type`
-    :param secondary_field: password. Used only if `user_identity_type` has
+    :ivar primary_field: user name or kerberos ticket depending on
+                         value of `user_identity_type`
+    :ivar secondary_field: password. Used only if `user_identity_type` has
                            value of 2
-    :param user_identity_type: type of user identification. Defaults to 2
-                               which is username/password identification
-    :param positive_response_req: 0 - no response requested,
-                                  1 - positive response requested
-    :param reserved: reserved field, defaults to 0x00. In most cases you
-                     should not change it's value or check it
+    :ivar user_identity_type: type of user identification. Defaults to 2
+                              which is username/password identification
+    :ivar positive_response_req: 0 - no response requested,
+                                 1 - positive response requested
+    :ivar reserved: reserved field, defaults to 0x00. In most cases you
+                    should not change it's value or check it
     """
     item_type = 0x58
     header = struct.Struct('>B B H B B H')
 
     def __init__(self, primary_field, secondary_field='', user_identity_type=2,
                  positive_response_req=0, reserved=0x00):
+        # type: (str,str,int,int,int) -> None
         """Initializes new sub item instance"""
         self.reserved = reserved  # byte
         self.user_identity_type = user_identity_type  # byte
         self.positive_response_req = positive_response_req
-        self.primary_field = primary_field  # string
-        self.secondary_field = secondary_field  # string
+        self._primary_field = primary_field.encode('utf8')  # string
+        self._secondary_field = secondary_field.encode('utf8')  # string
+
+    @property
+    def primary_field(self):
+        """Sub-item primary field value.
+
+        Meaning of the value depends on the `user_identity_type` value
+
+        :return: primary field value
+        :rtype: str
+        """
+        return self._primary_field.decode('utf8')
+
+    @property
+    def secondary_field(self):
+        """Sub-item secondary field value.
+
+        Meaning of the value depends on the `user_identity_type` value
+
+        :return: secondary field value
+        :rtype: str
+        """
+        return self._secondary_field.decode('utf8')
 
     def __repr__(self):
         return 'UserIdentityNegotiationSubItem(' \
@@ -309,7 +451,7 @@ class UserIdentityNegotiationSubItem(object):
 
         :return: item length
         """
-        return 6 + len(self.primary_field) + len(self.secondary_field)
+        return 6 + len(self._primary_field) + len(self._secondary_field)
 
     @property
     def total_length(self):
@@ -324,15 +466,13 @@ class UserIdentityNegotiationSubItem(object):
 
         :return: binary representation of an item
         """
-        primary_field = self.primary_field.encode()
-        secondary_field = self.secondary_field.encode()
         return b''.join(
             [self.header.pack(self.item_type, self.reserved, self.item_length,
                               self.user_identity_type,
                               self.positive_response_req,
-                              len(primary_field)),
-             primary_field, struct.pack('>H', len(secondary_field)),
-             secondary_field])
+                              len(self._primary_field)),
+             self._primary_field, struct.pack('>H', len(self._secondary_field)),
+             self._secondary_field])
 
     @classmethod
     def decode(cls, stream):
@@ -341,13 +481,13 @@ class UserIdentityNegotiationSubItem(object):
         :param stream: binary stream that should be decoded
         :return: new sub-item
         """
-        _, reserved, item_length, user_identity_type, \
+        _, reserved, _, user_identity_type, \
             positive_response_req, \
             primary_field_len = cls.header.unpack(stream.read(cls.header.size))
-        primary_field = stream.read(primary_field_len).decode()
+        primary_field = stream.read(primary_field_len)
         secondary_field_len = struct.unpack('>H', stream.read(2))[0]
-        secondary_field = stream.read(secondary_field_len).decode()
-        return cls(primary_field, secondary_field, user_identity_type,
+        secondary_field = stream.read(secondary_field_len)
+        return cls(primary_field.decode('utf8'), secondary_field.decode('utf8'), user_identity_type,
                    positive_response_req, reserved)
 
 
@@ -358,15 +498,16 @@ class UserIdentityNegotiationSubItemAc(object):
     Server response (accept) user identification sub-item. This item is
     expected only if `positive_response_req` was set to 1 in request sub-item.
 
-    :param server_response: kerberos service ticket or SAML response,
-                            depending on requested user identification type
-    :param reserved: reserved field, defaults to 0x00. In most cases you
-                     should not change it's value or check it
+    :ivar server_response: kerberos service ticket or SAML response,
+                           depending on requested user identification type
+    :ivar reserved: reserved field, defaults to 0x00. In most cases you
+                    should not change it's value or check it
     """
     item_type = 0x59
     header = struct.Struct('>B B H H')
 
     def __init__(self, server_response, reserved=0x00):
+        # type: (str,int) -> None
         """Initializes new response sub-item"""
         self.reserved = reserved  # byte
         self.server_response = server_response  # string
@@ -410,8 +551,7 @@ class UserIdentityNegotiationSubItemAc(object):
         :param stream: binary stream that should be decoded
         :return: new sub-item
         """
-        _, reserved, item_length, \
-            response_len = cls.header.unpack(stream.read(cls.header.size))
+        _, reserved, _, response_len = cls.header.unpack(stream.read(cls.header.size))
         server_response = stream.read(response_len).decode()
         return cls(server_response, reserved)
 
@@ -425,6 +565,7 @@ class GenericUserDataSubItem(object):
     header = struct.Struct('>B B H')
 
     def __init__(self, item_type, user_data, reserved=0x00):
+        # type: (int,bytes,int) -> None
         self.item_type = item_type  # unsigned byte
         self.reserved = reserved  # unsigned byte
         self.user_data = user_data  # raw string
@@ -436,16 +577,27 @@ class GenericUserDataSubItem(object):
 
     @property
     def item_length(self):
+        """Calculates item length
+
+        :return: item length
+        """
         return len(self.user_data)
 
     @property
     def total_length(self):
+        """Returns total item length, including header.
+
+        :return: total item length
+        """
         return 4 + self.item_length
 
     def encode(self):
-        return b''.join([self.header.pack(self.item_type, self.reserved,
-                                          self.item_length),
-                        self.user_data])
+        """Encodes itself into binary form
+
+        :return: binary representation of an item
+        """
+        return b''.join([self.header.pack(self.item_type, self.reserved, self.item_length),
+                         self.user_data])
 
     @classmethod
     def decode(cls, stream):
